import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Widget/filter_widget/util.dart';
import 'package:wowit/model.dart';
import 'package:wowit/sqlService.dart';

const String fieldPageNumber = 'PageNumber';
const String fieldRowsOfPage = 'RowsOfPage';
const String fieldOrderByField = 'orderByField';
const String fieldOrderByAsc = 'orderByAsc';

abstract class LoadMoreBaseController extends BaseController {
  Map<String, List<Map<String, Object>>> tempData = {};

  int get rowsOfPage => 10;

  int _currentPage = 1;
  int get currentPage => _currentPage;

  bool _hasNextPage = true;
  bool get hasNextPage => _hasNextPage;

  bool _isLoadMoreRunning = false;
  bool get isLoadMoreRunning => _isLoadMoreRunning;

  Map<String, Object> mapCondition = {};

  String orderByField;
  bool orderByASC;

  Future<Map<String, List<Map<String, Object>>>> execute(String query,
      {Map<String, Object> params}) async {
    params.addAll({
      "@$fieldPageNumber": _currentPage,
      "@$fieldOrderByField": orderByField,
      "@$fieldOrderByAsc": orderByASC,
      '@$fieldRowsOfPage': rowsOfPage,
    });

    mapCondition.forEach((key, value) {
      if (mapCondition[key] is RangeValue) {
        final giaRangeValue = value as RangeValue;
        params['@${key}Min'] = giaRangeValue.start;
        params['@${key}Max'] = (value as RangeValue).end;
      } else {
        params['@$key'] = value;
      }
    });

    print('params: $params');

    final data = await SQLService.execute(query, params: params);

    String displayTable = getFilterFieldName(data);

    if (data[displayTable].isEmpty) {
      _hasNextPage = false;
    }

    if (currentPage == 1) {
      tempData.clear();
      tempData.addEntries(data.entries);
    }

    return data;
  }

  void loadMore() async {
    print('call loadMore');

    if (hasNextPage && isLoadMoreRunning == false) {
      _currentPage++;
      _isLoadMoreRunning = true;
      update();

      print('${this.runtimeType} load more with page = $currentPage');

      final data = await loadData();
      String displayTable = getFilterFieldName(data);
      tempData[displayTable]?.addAll(data[displayTable]);
      setData(tempData);

      _isLoadMoreRunning = false;
      update();
    }
  }

  void sort(String orderByField, bool orderByAsc) async {
    this.orderByField = orderByField;
    this.orderByASC = orderByAsc;
    resetJParam();
    clearData();
    update();
  }

  void filter(Map<String, Object> mapCondition) async {
    this.mapCondition = mapCondition ?? {};
    resetJParam();
    clearData();
    update();
  }

  void resetJParam() {
    _currentPage = 1;
    _hasNextPage = true;
    _isLoadMoreRunning = false;
  }
}
