import 'package:flutter/rendering.dart';
import 'package:wowit/Controller/searched_controller.dart';
import 'package:wowit/Widget/filter_widget/component/seached_list.dart';
import 'package:wowit/Widget/filter_widget/filter_widget.dart';
import 'package:wowit/message.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class SearchScreen extends StatefulWidget {
  SearchScreen({
    Key key,
    this.searchedListModel,
    this.filterIcon,
    @required this.fieldNameSearching,
    @required this.loadSearchedData,
  })  : assert(fieldNameSearching != null,
            'FitlerWidgetException: fieldNameSearching must be provide'),
        assert(loadSearchedData != null,
            'FitlerWidgetException: Please implement SeachedController'),
        super(key: key);

  final SearchedDataLoader loadSearchedData;
  final String fieldNameSearching;
  final Widget filterIcon;

  final SearchedListModel searchedListModel;

  @override
  _SearchScreenState createState() => _SearchScreenState();
}

class _SearchScreenState extends State<SearchScreen> {
  final keywordController = TextEditingController();
  final ScrollController scrollController = ScrollController();
  SearchedList wSearchedList;

  void searchHome(String query) {
    wSearchedList.search(query);
  }

  @override
  void initState() {
    final filterItems = <FilterItem>[];

    filterItems.addAll(widget.searchedListModel.filterItems);

    filterItems.add(
      FilterItem(
        fieldName: widget.fieldNameSearching,
        filterMode: EFilterMode.insensitiveAndUnsignedContain,
      ),
    );

    wSearchedList = SearchedList(
      searchedListModel: widget.searchedListModel,
      fieldNameSearching: widget.fieldNameSearching,
      searchedDataLoader: widget.loadSearchedData,
    );

    wSearchedList.scrollController = scrollController;

    scrollController.addListener(() {
      if (scrollController.position.extentAfter < 200 &&
          scrollController.position.userScrollDirection ==
              ScrollDirection.reverse) {
        wSearchedList.loadMore?.call();
      }
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (wSearchedList.getValueConditionByFieldName(widget.fieldNameSearching) ==
        null) {
      wSearchedList.search(keywordController.text);
    }
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: _buildAppBar(context),
      body: wSearchedList,
    );
  }

  AppBar _buildAppBar(BuildContext context) {
    return AppBar(
      leading: IconButton(
        onPressed: () {
          FocusScope.of(context).requestFocus(FocusNode());
          Get.back();
        },
        icon: Icon(
          Icons.arrow_back_ios_outlined,
          color: Color(0xff2a367e),
        ),
        splashRadius: 24,
      ),
      titleSpacing: 0.0,
      elevation: 0.5,
      title: _SearchTextField(
        controller: keywordController,
        onChanged: searchHome,
        filterIcon: widget.filterIcon,
        goToFilterView: wSearchedList.goToFilterView,
      ),
      backgroundColor: Colors.white,
    );
  }
}

class _SearchTextField extends StatefulWidget {
  final TextEditingController controller;
  final ValueChanged<String> onChanged;
  final VoidCallback goToFilterView;
  final Widget filterIcon;
  const _SearchTextField(
      {Key key,
      this.controller,
      this.onChanged,
      this.goToFilterView,
      this.filterIcon})
      : super(key: key);

  @override
  _SearchTextFieldState createState() => _SearchTextFieldState();
}

class _SearchTextFieldState extends State<_SearchTextField> {
  bool isShowCloseButton = false;
  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        SizedBox(width: 16),
        Expanded(
          child: TextField(
            autofocus: true,
            controller: widget.controller,
            onChanged: (value) {
              widget.onChanged(value);
              if (isShowCloseButton && value.isEmpty) {
                setState(() {
                  isShowCloseButton = false;
                });
              } else if (!isShowCloseButton && value.isNotEmpty) {
                setState(() {
                  isShowCloseButton = true;
                });
              }
            },
            decoration: InputDecoration(
              border: InputBorder.none,
              hintText: Message.getMessage('HOME_Search_Tim_Kiem'),
            ),
          ),
        ),
        if (isShowCloseButton)
          IconButton(
            padding: EdgeInsets.all(4.0),
            constraints: BoxConstraints(),
            onPressed: () {
              widget.controller.clear();
              widget.onChanged('');
              setState(() {
                isShowCloseButton = false;
              });
            },
            icon: Icon(
              Icons.close,
              color: Color(0xff2a367e),
            ),
            splashRadius: 24,
          ),
        IconButton(
          padding: EdgeInsets.zero,
          onPressed: () {
            FocusScope.of(context).requestFocus(FocusNode());
            widget.goToFilterView();
          },
          icon: widget.filterIcon ??
              Icon(
                Icons.filter_alt_outlined,
                color: Color(0xff2a367e),
              ),
          splashRadius: 24,
        ),
      ],
    );
  }
}
